﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using Windows.Foundation;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Input.Inking;
using Windows.UI;
using System.Text;

namespace InkRecoApp
{
    /// <summary>
    /// 可用于自身或导航至 Frame 内部的空白页。
    /// </summary>
    public sealed partial class MainPage : Page
    {
        InkRecognizerContainer inkRecognContainer = new InkRecognizerContainer();
        DispatcherTimer timer = new DispatcherTimer();
        public MainPage()
        {
            this.InitializeComponent();

            // 获取已安装的识别引擎列表
            var inkrecogs = inkRecognContainer.GetRecognizers();
            // 将这些列表显示到ComboBox控件中
            cmbRecons.ItemsSource = inkrecogs;
            // 处理选项更改事件
            cmbRecons.SelectionChanged += (s1, s2) =>
            {
                // 将选中的识别引擎设为默认
                InkRecognizer currec = (InkRecognizer)cmbRecons.SelectedItem;
                inkRecognContainer.SetDefaultRecognizer(currec);
            };
            if (cmbRecons.Items.Count > 0)
                cmbRecons.SelectedIndex = 0;
            // 全能书写
            inkcv.InkPresenter.InputDeviceTypes = Windows.UI.Core.CoreInputDeviceTypes.Mouse | Windows.UI.Core.CoreInputDeviceTypes.Touch | Windows.UI.Core.CoreInputDeviceTypes.Pen;
            // 设置好InkCanvas的外观
            var attr = inkcv.InkPresenter.CopyDefaultDrawingAttributes();
            attr.Color = Colors.Green;
            attr.Size = new Size(5d, 5d); //笔头大小
            attr.PenTip = PenTipShape.Circle; //圆笔尖
            inkcv.InkPresenter.UpdateDefaultDrawingAttributes(attr);

            // 准备计时器
            // 延迟2秒，应该不算慢吧
            timer.Interval = TimeSpan.FromSeconds(2d);
            timer.Tick += onTimerTick;
            // 处理ink操作事件
            inkcv.InkPresenter.StrokeInput.StrokeStarted += (k1, k2) =>
            {
                // 人家正要下笔呢，没有在此时识别的道理
                timer.Stop();
            };
            inkcv.InkPresenter.StrokesCollected += (t1, t2) =>
            {
                // 墨迹已收集，可以进行识别
                timer.Start();
            };
        }

        private async void onTimerTick(object sender, object e)
        {
            // 如果InkStrokeContainer中没有收集笔触，那就没有识别的必要了
            // 所以Count应大于0
            if (inkcv.InkPresenter.StrokeContainer.GetStrokes().Count > 0)
            {
                IReadOnlyList<InkRecognitionResult> results = await inkRecognContainer.RecognizeAsync(inkcv.InkPresenter.StrokeContainer, InkRecognitionTarget.All);
                // 处理结果
                if (results.Count > 0)
                {
                    StringBuilder strbd = new StringBuilder();
                    strbd.AppendLine("结果：");
                    // 每个InkRecognitionResult实例表示一个汉字/单词的识别结果
                    // 而单个结果中又包含候选列表，最接近的识别结果优先级更高
                    for(int x = 0; x < results.Count; x++)
                    {
                        string s = string.Join("，", results[x].GetTextCandidates().ToArray());
                        strbd.AppendLine(s);
                    }
                    // 显示结果
                    tbresult.Text = strbd.ToString();
                    // 清理墨迹
                    inkcv.InkPresenter.StrokeContainer.Clear();
                }
            }
        }
    }
}
